/* $Id: QuickReferenceServlet.java,v 1.1 2009/09/16 08:29:20 stm Exp $
 *
 * PDFlib+PDI client: mini imposition demo
 */

import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.pdflib.pdflib;
import com.pdflib.PDFlibException;

public class QuickReferenceServlet extends HttpServlet {
    private static final long serialVersionUID = 8894087547592610154L;

    public void service(HttpServletRequest request, HttpServletResponse response)
            throws ServletException {
        int font, row = 0, col = 0, i;
        int manual, pages;
        final int maxrow = 2, maxcol = 2;
        int endpage;
        final float width = 500, height = 770;
        int pageno;
        String infile = "reference.pdf";

        /* This is where font/image/PDF input files live. Adjust as necessary. */
        ServletContext context = getServletContext();
        String searchpath = context.getRealPath("/WEB-INF/data");

        if (searchpath == null) {
            throw new ServletException(
                    "Error: Unable to access application data (WAR file not unpacked?)");
        }

        pdflib p = null;

        try {
            p = new pdflib();

            // Generate a PDF in memory; insert a file name to create PDF on
            // disk
            if (p.begin_document("", "") == -1) {
                throw new ServletException("Error: " + p.get_apiname() + ": " + p.get_errmsg());
            }

            p.set_parameter("SearchPath", searchpath);

            p.set_info("Creator", "quickreferenceServlet.java");
            p.set_info("Author", "Rainer Ploeckl");
            p.set_info("Title", "imposition demo (Java/Servlet)");

            manual = p.open_pdi_document(infile, "");
            i = 0;

            if (manual == -1) {
                throw new ServletException("Error: " + p.get_apiname() + ": " + p.get_errmsg());
            }

            p.set_parameter("topdown", "true");

            endpage = (int) p.pcos_get_number(manual, "length:pages");

            for (pageno = 1; pageno <= endpage; pageno++) {
                if (row == 0 && col == 0) {
                    i++;
                    p.begin_page_ext(width, height, "");
                    font = p.load_font("Helvetica-Bold", "unicode", "");
                    p.setfont(font, 18);
                    p.set_text_pos(24, 24);
                    p.show("PDFlib Quick Reference");
                }

                pages = p.open_pdi_page(manual, pageno, "");

                if (pages == -1) {
                    throw new ServletException("Error: " + p.get_apiname()
                            + ": " + p.get_errmsg());
                }

                p.fit_pdi_page(manual, width / maxcol * col, (row + 1) * height
                        / maxrow, "scale " + (float) 1 / maxrow);
                p.close_pdi_page(pages);

                col++;
                if (col == maxcol) {
                    col = 0;
                    row++;
                }
                if (row == maxrow) {
                    row = 0;
                    p.end_page_ext("");
                }
            }

            // finish the last partial page
            if (row != 0 || col != 0)
                p.end_page_ext("");

            p.end_document("");
            p.close_pdi_document(manual);

            byte[] buf = p.get_buffer();

            response.setContentType("application/pdf");
            response.setContentLength(buf.length);
            response.setHeader("Content-Disposition",
                "inline; filename=quickreference.pdf");

            ServletOutputStream output = response.getOutputStream();
            output.write(buf);
            output.close();
        }
        catch (PDFlibException e) {
            throw new ServletException("PDFlib exception occurred in "
                    + getClass().getName(), e);
        }
        catch (Exception e) {
            throw new ServletException("Exception occurred in "
                    + getClass().getName(), e);
        }
        finally {
            if (p != null) {
                p.delete(); /* delete the PDFlib object */
            }
        }
    }
}
